@@ -1,4 +1,5 @@
 utils::globalVariables(c("libsizes","x_xmap_y_mean", "y_xmap_x_mean",
                          "x_xmap_y_lower", "x_xmap_y_upper",
                          "y_xmap_x_lower", "y_xmap_x_upper",
-                         "x_xmap_y_sig", "y_xmap_x_sig"))
+                         "x_xmap_y_sig", "y_xmap_x_sig",
+                         "causality", "type", "q50"))
