@@ -10,7 +10,7 @@
 #include <unordered_set>
 #include "CppStats.h"
 #include "CppDistances.h"
-#include "spEDMDataStruct.h"
+#include "DataStruct.h"
 #include "IntersectionCardinality.h"
 #include <RcppThread.h>
 
