@@ -34,6 +34,7 @@
  * @param row_size_mark        If true, use the row-wise libsize to mark the libsize; if false, use col-wise libsize.
  * @param dist_metric          Distance metric selector (1: Manhattan, 2: Euclidean).
  * @param dist_average         Whether to average distance by the number of valid vector components.
+ * @param win_ratios           Scale the sliding window step relative to the matrix width/height to speed up state-space predictions.
  *
  * @return  A vector of pairs, where each pair contains the library size and the corresponding cross mapping result.
  */
@@ -52,7 +53,8 @@ std::vector<std::pair<int, double>> GCCMSingle4Grid(
     int parallel_level,
     bool row_size_mark,
     int dist_metric,
-    bool dist_average
+    bool dist_average,
+    const std::vector<double>& win_ratios = {0,0}
 ) {
   // Extract row-wise and column-wise library sizes
   const int lib_size_row = lib_sizes[0];
@@ -61,10 +63,112 @@ std::vector<std::pair<int, double>> GCCMSingle4Grid(
   // Determine the marked libsize
   const int libsize = row_size_mark ? lib_size_row : lib_size_col;
 
+  // Computate step of sliding windows
+  const int r_step = std::max(1, static_cast<int>(std::round(1.0 + win_ratios[0] * lib_size_row)));
+  const int c_step = std::max(1, static_cast<int>(std::round(1.0 + win_ratios[1] * lib_size_col)));
+
   // Precompute valid (r, c) pairs
   std::vector<std::pair<int, int>> valid_indices;
-  for (int r = 1; r <= totalRow - lib_size_row + 1; ++r) {
-    for (int c = 1; c <= totalCol - lib_size_col + 1; ++c) {
+  // // Slide the window by 1 row and 1 column
+  // for (int r = 1; r <= totalRow - lib_size_row + 1; ++r) {
+  //   for (int c = 1; c <= totalCol - lib_size_col + 1; ++c) {
+  //     valid_indices.emplace_back(r, c);
+  //   }
+  // }
+  // Slide the window by flexible row and column
+  for (int r = 1; r <= totalRow - lib_size_row + 1; r += r_step) {
+    for (int c = 1; c <= totalCol - lib_size_col + 1; c += c_step) {
+      valid_indices.emplace_back(r, c);
+    }
+  }
+
+  // // Initialize the result container with the same size as valid_indices
+  // std::vector<std::pair<int, double>> x_xmap_y;
+  // x_xmap_y.resize(valid_indices.size());
+
+  // Preallocate the result vector to avoid out-of-bounds access
+  std::vector<std::pair<int, double>> x_xmap_y(valid_indices.size());
+
+  // Unified processing logic
+  auto process = [&](size_t idx) {
+    const int r = valid_indices[idx].first;
+    const int c = valid_indices[idx].second;
+
+    // Initialize library indices and count the number of the nan value together
+    std::vector<int> lib_indices;
+
+    for (int ii = r; ii < r + lib_size_row; ++ii) {
+      for (int jj = c; jj < c + lib_size_col; ++jj) {
+        int index = (ii - 1) * totalCol + (jj - 1);
+        if (possible_lib_indices[index]) {
+          lib_indices.emplace_back(index);
+        }
+      }
+    }
+
+    double rho = std::numeric_limits<double>::quiet_NaN();
+    // Run cross map and store results
+    if (simplex) {
+      rho = SimplexProjection(xEmbedings, yPred, lib_indices, pred_indices, b, dist_metric, dist_average);
+    } else {
+      rho = SMap(xEmbedings, yPred, lib_indices, pred_indices, b, theta, dist_metric, dist_average);
+    }
+    x_xmap_y[idx] = std::make_pair(libsize, rho);
+  };
+
+  // Parallel coordination
+  if (parallel_level == 0) {
+    RcppThread::parallelFor(0, valid_indices.size(), process, threads);
+  } else {
+    for (size_t i = 0; i < valid_indices.size(); ++i) {
+      process(i);
+    }
+  }
+
+  return x_xmap_y;
+}
+
+// Perform Grid-based Geographical Convergent Cross Mapping (GCCM) for a single library size and pred indice (composite embeddings version).
+std::vector<std::pair<int, double>> GCCMSingle4Grid(
+    const std::vector<std::vector<std::vector<double>>>& xEmbedings,
+    const std::vector<double>& yPred,
+    const std::vector<int>& lib_sizes,
+    const std::vector<bool>& possible_lib_indices,
+    const std::vector<int>& pred_indices,
+    int totalRow,
+    int totalCol,
+    int b,
+    bool simplex,
+    double theta,
+    size_t threads,
+    int parallel_level,
+    bool row_size_mark,
+    int dist_metric,
+    bool dist_average,
+    const std::vector<double>& win_ratios = {0,0}
+) {
+  // Extract row-wise and column-wise library sizes
+  const int lib_size_row = lib_sizes[0];
+  const int lib_size_col = lib_sizes[1];
+
+  // Determine the marked libsize
+  const int libsize = row_size_mark ? lib_size_row : lib_size_col;
+
+  // Computate step of sliding windows
+  const int r_step = std::max(1, static_cast<int>(std::round(1.0 + win_ratios[0] * lib_size_row)));
+  const int c_step = std::max(1, static_cast<int>(std::round(1.0 + win_ratios[1] * lib_size_col)));
+
+  // Precompute valid (r, c) pairs
+  std::vector<std::pair<int, int>> valid_indices;
+  // // Slide the window by 1 row and 1 column
+  // for (int r = 1; r <= totalRow - lib_size_row + 1; ++r) {
+  //   for (int c = 1; c <= totalCol - lib_size_col + 1; ++c) {
+  //     valid_indices.emplace_back(r, c);
+  //   }
+  // }
+  // Slide the window by flexible row and column
+  for (int r = 1; r <= totalRow - lib_size_row + 1; r += r_step) {
+    for (int c = 1; c <= totalCol - lib_size_col + 1; c += c_step) {
       valid_indices.emplace_back(r, c);
     }
   }
@@ -252,6 +356,121 @@ std::vector<std::pair<int, double>> GCCMSingle4GridOneDim(
   }
 }
 
+// Perform Grid-based Geographical Convergent Cross Mapping (GCCM) for a single library size (composite embeddings version).
+std::vector<std::pair<int, double>> GCCMSingle4GridOneDim(
+    const std::vector<std::vector<std::vector<double>>>& xEmbedings,
+    const std::vector<double>& yPred,
+    int lib_size,
+    const std::vector<int>& lib_indices,
+    const std::vector<int>& pred_indices,
+    int totalRow,
+    int totalCol,
+    int b,
+    bool simplex,
+    double theta,
+    size_t threads,
+    int parallel_level,
+    int dist_metric,
+    bool dist_average
+) {
+  int max_lib_size = lib_indices.size();
+
+  // No possible library variation if using all vectors
+  if (lib_size == max_lib_size) {
+    std::vector<std::pair<int, double>> x_xmap_y;
+
+    double rho = std::numeric_limits<double>::quiet_NaN();
+    // Run cross map and store results
+    if (simplex) {
+      rho = SimplexProjection(xEmbedings, yPred, lib_indices, pred_indices, b, dist_metric, dist_average);
+    } else {
+      rho = SMap(xEmbedings, yPred, lib_indices, pred_indices, b, theta, dist_metric, dist_average);
+    }
+    x_xmap_y.emplace_back(lib_size, rho);
+    return x_xmap_y;
+  } else if (parallel_level == 0){
+    // Precompute valid indices for the library
+    std::vector<std::vector<int>> valid_lib_indices;
+    for (int start_lib = 0; start_lib < max_lib_size; ++start_lib) {
+      std::vector<int> local_lib_indices;
+      // Loop around to beginning of lib indices
+      if (start_lib + lib_size > max_lib_size) {
+        for (int i = start_lib; i < max_lib_size; ++i) {
+          local_lib_indices.emplace_back(lib_indices[i]);
+        }
+        int num_vectors_remaining = lib_size - (max_lib_size - start_lib);
+        for (int i = 0; i < num_vectors_remaining; ++i) {
+          local_lib_indices.emplace_back(lib_indices[i]);
+        }
+      } else {
+        for (int i = start_lib; i < start_lib + lib_size; ++i) {
+          local_lib_indices.emplace_back(lib_indices[i]);
+        }
+      }
+      valid_lib_indices.emplace_back(local_lib_indices);
+    }
+
+    // Preallocate the result vector to avoid out-of-bounds access
+    std::vector<std::pair<int, double>> x_xmap_y(valid_lib_indices.size());
+
+    // Perform the operations using RcppThread
+    RcppThread::parallelFor(0, valid_lib_indices.size(), [&](size_t i) {
+      double rho = std::numeric_limits<double>::quiet_NaN();
+      // Run cross map and store results
+      if (simplex) {
+        rho = SimplexProjection(xEmbedings, yPred, valid_lib_indices[i], pred_indices, b, dist_metric, dist_average);
+      } else {
+        rho = SMap(xEmbedings, yPred, valid_lib_indices[i], pred_indices, b, theta, dist_metric, dist_average);
+      }
+
+      std::pair<int, double> result(lib_size, rho); // Store the product of row and column library sizes
+      x_xmap_y[i] = result;
+    }, threads);
+
+    return x_xmap_y;
+  } else {
+    // Precompute valid indices for the library
+    std::vector<std::vector<int>> valid_lib_indices;
+    for (int start_lib = 0; start_lib < max_lib_size; ++start_lib) {
+      std::vector<int> local_lib_indices;
+      // Loop around to beginning of lib indices
+      if (start_lib + lib_size > max_lib_size) {
+        for (int i = start_lib; i < max_lib_size; ++i) {
+          local_lib_indices.emplace_back(lib_indices[i]);
+        }
+        int num_vectors_remaining = lib_size - (max_lib_size - start_lib);
+        for (int i = 0; i < num_vectors_remaining; ++i) {
+          local_lib_indices.emplace_back(lib_indices[i]);
+        }
+      } else {
+        for (int i = start_lib; i < start_lib + lib_size; ++i) {
+          local_lib_indices.emplace_back(lib_indices[i]);
+        }
+      }
+      valid_lib_indices.emplace_back(local_lib_indices);
+    }
+
+    // Preallocate the result vector to avoid out-of-bounds access
+    std::vector<std::pair<int, double>> x_xmap_y(valid_lib_indices.size());
+
+    // Iterate through precomputed valid_lib_indices
+    for (size_t i = 0; i < valid_lib_indices.size(); ++i){
+      double rho = std::numeric_limits<double>::quiet_NaN();
+      // Run cross map and store results
+      if (simplex) {
+        rho = SimplexProjection(xEmbedings, yPred, valid_lib_indices[i], pred_indices, b, dist_metric, dist_average);
+      } else {
+        rho = SMap(xEmbedings, yPred, valid_lib_indices[i], pred_indices, b, theta, dist_metric, dist_average);
+      }
+
+      std::pair<int, double> result(lib_size, rho); // Store the product of row and column library sizes
+      x_xmap_y[i] = result;
+    }
+
+    return x_xmap_y;
+  }
+}
+
 /**
  * Perform Geographical Convergent Cross Mapping (GCCM) for spatial grid data.
  *
@@ -271,9 +490,12 @@ std::vector<std::pair<int, double>> GCCMSingle4GridOneDim(
  * @param threads        The number of threads to use for parallel processing.
  * @param parallel_level Level of parallel computing: 0 for `lower`, 1 for `higher`.
  * @param style          Embedding style selector (0: includes current state, 1: excludes it).
+ * @param stack          Embedding arrangement selector (0: single - average lags, 1: composite - stack).  Default is 0 (average lags).
  * @param dist_metric    Distance metric selector (1: Manhattan, 2: Euclidean).
  * @param dist_average   Whether to average distance by the number of valid vector components.
  * @param single_sig     Whether to estimate significance and confidence intervals using a single rho value.
+ * @param dir            Direction selector for embeddings where 0 returns all directions for embeddings, 1–8 correspond to NW, N, NE, W, E, SW, S, SE, and multiple directions can be combined (e.g., {1,2,3} for NW, N, NE).
+ * @param win_rations    Scale the sliding window step relative to the matrix width/height to speed up state-space predictions.
  * @param progressbar    If true, display a progress bar during computation.
  *
  * @return A 2D vector where each row contains the library size, mean cross mapping result,
@@ -293,10 +515,13 @@ std::vector<std::vector<double>> GCCM4Grid(
     int threads,
     int parallel_level,
     int style,
+    int stack,
     int dist_metric,
     bool dist_average,
     bool single_sig,
-    bool progressbar
+    const std::vector<int>& dir = {0},
+    const std::vector<double>& win_ratios = {0,0},
+    bool progressbar = false
 ) {
   // If b is not provided correctly, default it to E + 2
   if (b <= 0) {
@@ -318,7 +543,13 @@ std::vector<std::vector<double>> GCCM4Grid(
   }
 
   // Generate embeddings for xMatrix
-  std::vector<std::vector<double>> xEmbedings = GenGridEmbeddings(xMatrix, E, tau, style);
+  std::vector<std::vector<double>> Emb2D;
+  std::vector<std::vector<std::vector<double>>> Emb3D;
+  if (stack == 0){
+    Emb2D = GenGridEmbeddings(xMatrix, E, tau, style);
+  } else {
+    Emb3D = GenGridEmbeddingsCom(xMatrix, E, tau, style, dir);
+  }
 
   // Ensure the maximum value does not exceed totalRow or totalCol
   int max_lib_size_row = totalRow;
@@ -399,44 +630,86 @@ std::vector<std::vector<double>> GCCM4Grid(
       for (size_t i = 0; i < unique_lib_size_pairs.size(); ++i) {
         int lib_size_row = unique_lib_size_pairs[i].first;
         int lib_size_col = unique_lib_size_pairs[i].second;
-        local_results[i] = GCCMSingle4Grid(
-          xEmbedings,
-          yPred,
-          {lib_size_row, lib_size_col},
-          lib_indices,
-          pred_indices,
-          totalRow,
-          totalCol,
-          b,
-          simplex,
-          theta,
-          threads_sizet,
-          parallel_level,
-          row_size_mark,
-          dist_metric,
-          dist_average);
+        if (stack == 0){
+          local_results[i] = GCCMSingle4Grid(
+            Emb2D,
+            yPred,
+            {lib_size_row, lib_size_col},
+            lib_indices,
+            pred_indices,
+            totalRow,
+            totalCol,
+            b,
+            simplex,
+            theta,
+            threads_sizet,
+            parallel_level,
+            row_size_mark,
+            dist_metric,
+            dist_average,
+            win_ratios);
+        } else {
+          local_results[i] = GCCMSingle4Grid(
+            Emb3D,
+            yPred,
+            {lib_size_row, lib_size_col},
+            lib_indices,
+            pred_indices,
+            totalRow,
+            totalCol,
+            b,
+            simplex,
+            theta,
+            threads_sizet,
+            parallel_level,
+            row_size_mark,
+            dist_metric,
+            dist_average,
+            win_ratios);
+        }
         bar++;
       }
     } else {
       for (size_t i = 0; i < unique_lib_size_pairs.size(); ++i) {
         int lib_size_row = unique_lib_size_pairs[i].first;
         int lib_size_col = unique_lib_size_pairs[i].second;
-        local_results[i] = GCCMSingle4Grid(
-          xEmbedings,
-          yPred,
-          {lib_size_row, lib_size_col},
-          lib_indices,
-          pred_indices,
-          totalRow,
-          totalCol,
-          b,
-          simplex,
-          theta,
-          threads_sizet,
-          parallel_level,
-          row_size_mark,
-          dist_metric,
-          dist_average);
+        if (stack == 0){
+          local_results[i] = GCCMSingle4Grid(
+            Emb2D,
+            yPred,
+            {lib_size_row, lib_size_col},
+            lib_indices,
+            pred_indices,
+            totalRow,
+            totalCol,
+            b,
+            simplex,
+            theta,
+            threads_sizet,
+            parallel_level,
+            row_size_mark,
+            dist_metric,
+            dist_average,
+            win_ratios);
+        } else {
+          local_results[i] = GCCMSingle4Grid(
+            Emb3D,
+            yPred,
+            {lib_size_row, lib_size_col},
+            lib_indices,
+            pred_indices,
+            totalRow,
+            totalCol,
+            b,
+            simplex,
+            theta,
+            threads_sizet,
+            parallel_level,
+            row_size_mark,
+            dist_metric,
+            dist_average,
+            win_ratios);
+        }
       }
     }
   } else {
@@ -446,44 +719,86 @@ std::vector<std::vector<double>> GCCM4Grid(
       RcppThread::parallelFor(0, unique_lib_size_pairs.size(), [&](size_t i) {
         int lib_size_row = unique_lib_size_pairs[i].first;
         int lib_size_col = unique_lib_size_pairs[i].second;
-        local_results[i] = GCCMSingle4Grid(
-          xEmbedings,
-          yPred,
-          {lib_size_row, lib_size_col},
-          lib_indices,
-          pred_indices,
-          totalRow,
-          totalCol,
-          b,
-          simplex,
-          theta,
-          threads_sizet,
-          parallel_level,
-          row_size_mark,
-          dist_metric,
-          dist_average);
+        if (stack == 0){
+          local_results[i] = GCCMSingle4Grid(
+            Emb2D,
+            yPred,
+            {lib_size_row, lib_size_col},
+            lib_indices,
+            pred_indices,
+            totalRow,
+            totalCol,
+            b,
+            simplex,
+            theta,
+            threads_sizet,
+            parallel_level,
+            row_size_mark,
+            dist_metric,
+            dist_average,
+            win_ratios);
+        } else {
+          local_results[i] = GCCMSingle4Grid(
+            Emb3D,
+            yPred,
+            {lib_size_row, lib_size_col},
+            lib_indices,
+            pred_indices,
+            totalRow,
+            totalCol,
+            b,
+            simplex,
+            theta,
+            threads_sizet,
+            parallel_level,
+            row_size_mark,
+            dist_metric,
+            dist_average,
+            win_ratios);
+        }
         bar++;
       }, threads_sizet);
     } else {
       RcppThread::parallelFor(0, unique_lib_size_pairs.size(), [&](size_t i) {
         int lib_size_row = unique_lib_size_pairs[i].first;
         int lib_size_col = unique_lib_size_pairs[i].second;
-        local_results[i] = GCCMSingle4Grid(
-          xEmbedings,
-          yPred,
-          {lib_size_row, lib_size_col},
-          lib_indices,
-          pred_indices,
-          totalRow,
-          totalCol,
-          b,
-          simplex,
-          theta,
-          threads_sizet,
-          parallel_level,
-          row_size_mark,
-          dist_metric,
-          dist_average);
+        if (stack == 0){
+          local_results[i] = GCCMSingle4Grid(
+            Emb2D,
+            yPred,
+            {lib_size_row, lib_size_col},
+            lib_indices,
+            pred_indices,
+            totalRow,
+            totalCol,
+            b,
+            simplex,
+            theta,
+            threads_sizet,
+            parallel_level,
+            row_size_mark,
+            dist_metric,
+            dist_average,
+            win_ratios);
+        } else {
+          local_results[i] = GCCMSingle4Grid(
+            Emb3D,
+            yPred,
+            {lib_size_row, lib_size_col},
+            lib_indices,
+            pred_indices,
+            totalRow,
+            totalCol,
+            b,
+            simplex,
+            theta,
+            threads_sizet,
+            parallel_level,
+            row_size_mark,
+            dist_metric,
+            dist_average,
+            win_ratios);
+        }
       }, threads_sizet);
     }
   }
@@ -567,9 +882,11 @@ std::vector<std::vector<double>> GCCM4Grid(
  * @param threads        The number of threads to use for parallel processing.
  * @param parallel_level Level of parallel computing: 0 for `lower`, 1 for `higher`.
  * @param style          Embedding style selector (0: includes current state, 1: excludes it).
+ * @param stack          Embedding arrangement selector (0: single - average lags, 1: composite - stack).  Default is 0 (average lags).
  * @param dist_metric    Distance metric selector (1: Manhattan, 2: Euclidean).
  * @param dist_average   Whether to average distance by the number of valid vector components.
  * @param single_sig     Whether to estimate significance and confidence intervals using a single rho value.
+ * @param dir            Direction selector for embeddings where 0 returns all directions, 1–8 correspond to NW, N, NE, W, E, SW, S, SE, and multiple directions can be combined (e.g., {1,2,3} for NW, N, NE).
  * @param progressbar    If true, display a progress bar during computation.
  *
  * @return A 2D vector where each row contains the library size, mean cross mapping result,
@@ -589,10 +906,12 @@ std::vector<std::vector<double>> GCCM4GridOneDim(
     int threads,
     int parallel_level,
     int style,
+    int stack,
     int dist_metric,
     bool dist_average,
     bool single_sig,
-    bool progressbar
+    const std::vector<int>& dir = {0},
+    bool progressbar = false
 ) {
   // If b is not provided correctly, default it to E + 2
   if (b <= 0) {
@@ -614,7 +933,13 @@ std::vector<std::vector<double>> GCCM4GridOneDim(
   }
 
   // Generate embeddings for xMatrix
-  std::vector<std::vector<double>> xEmbedings = GenGridEmbeddings(xMatrix, E, tau, style);
+  std::vector<std::vector<double>> Emb2D;
+  std::vector<std::vector<std::vector<double>>> Emb3D;
+  if (stack == 0){
+    Emb2D = GenGridEmbeddings(xMatrix, E, tau, style);
+  } else {
+    Emb3D = GenGridEmbeddingsCom(xMatrix, E, tau, style, dir);
+  }
 
   int max_lib_size = static_cast<int>(lib.size()); // Maximum lib size
 
@@ -640,42 +965,80 @@ std::vector<std::vector<double>> GCCM4GridOneDim(
     if (progressbar) {
       RcppThread::ProgressBar bar(unique_lib_sizes.size(), 1);
       for (size_t i = 0; i < unique_lib_sizes.size(); ++i) {
-        local_results[i] = GCCMSingle4GridOneDim(
-          xEmbedings,
-          yPred,
-          unique_lib_sizes[i],
-          lib,
-          pred,
-          totalRow,
-          totalCol,
-          b,
-          simplex,
-          theta,
-          threads_sizet,
-          parallel_level,
-          dist_metric,
-          dist_average
-        );
+        if (stack == 0){
+          local_results[i] = GCCMSingle4GridOneDim(
+            Emb2D,
+            yPred,
+            unique_lib_sizes[i],
+            lib,
+            pred,
+            totalRow,
+            totalCol,
+            b,
+            simplex,
+            theta,
+            threads_sizet,
+            parallel_level,
+            dist_metric,
+            dist_average
+          );
+        } else {
+          local_results[i] = GCCMSingle4GridOneDim(
+            Emb3D,
+            yPred,
+            unique_lib_sizes[i],
+            lib,
+            pred,
+            totalRow,
+            totalCol,
+            b,
+            simplex,
+            theta,
+            threads_sizet,
+            parallel_level,
+            dist_metric,
+            dist_average
+          );
+        }
         bar++;
       }
     } else {
       for (size_t i = 0; i < unique_lib_sizes.size(); ++i) {
-        local_results[i] = GCCMSingle4GridOneDim(
-          xEmbedings,
-          yPred,
-          unique_lib_sizes[i],
-          lib,
-          pred,
-          totalRow,
-          totalCol,
-          b,
-          simplex,
-          theta,
-          threads_sizet,
-          parallel_level,
-          dist_metric,
-          dist_average
-        );
+        if (stack == 0){
+          local_results[i] = GCCMSingle4GridOneDim(
+            Emb2D,
+            yPred,
+            unique_lib_sizes[i],
+            lib,
+            pred,
+            totalRow,
+            totalCol,
+            b,
+            simplex,
+            theta,
+            threads_sizet,
+            parallel_level,
+            dist_metric,
+            dist_average
+          );
+        } else {
+          local_results[i] = GCCMSingle4GridOneDim(
+            Emb3D,
+            yPred,
+            unique_lib_sizes[i],
+            lib,
+            pred,
+            totalRow,
+            totalCol,
+            b,
+            simplex,
+            theta,
+            threads_sizet,
+            parallel_level,
+            dist_metric,
+            dist_average
+          );
+        }
       }
     }
   } else {
@@ -684,43 +1047,81 @@ std::vector<std::vector<double>> GCCM4GridOneDim(
       RcppThread::ProgressBar bar(unique_lib_sizes.size(), 1);
       RcppThread::parallelFor(0, unique_lib_sizes.size(), [&](size_t i) {
         int lib_size = unique_lib_sizes[i];
-        local_results[i] = GCCMSingle4GridOneDim(
-          xEmbedings,
-          yPred,
-          lib_size,
-          lib,
-          pred,
-          totalRow,
-          totalCol,
-          b,
-          simplex,
-          theta,
-          threads_sizet,
-          parallel_level,
-          dist_metric,
-          dist_average
-        );
+        if (stack == 0){
+          local_results[i] = GCCMSingle4GridOneDim(
+            Emb2D,
+            yPred,
+            lib_size,
+            lib,
+            pred,
+            totalRow,
+            totalCol,
+            b,
+            simplex,
+            theta,
+            threads_sizet,
+            parallel_level,
+            dist_metric,
+            dist_average
+          );
+        } else {
+          local_results[i] = GCCMSingle4GridOneDim(
+            Emb3D,
+            yPred,
+            lib_size,
+            lib,
+            pred,
+            totalRow,
+            totalCol,
+            b,
+            simplex,
+            theta,
+            threads_sizet,
+            parallel_level,
+            dist_metric,
+            dist_average
+          );
+        }
         bar++;
       }, threads_sizet);
     } else {
       RcppThread::parallelFor(0, unique_lib_sizes.size(), [&](size_t i) {
         int lib_size = unique_lib_sizes[i];
-        local_results[i] = GCCMSingle4GridOneDim(
-          xEmbedings,
-          yPred,
-          lib_size,
-          lib,
-          pred,
-          totalRow,
-          totalCol,
-          b,
-          simplex,
-          theta,
-          threads_sizet,
-          parallel_level,
-          dist_metric,
-          dist_average
-        );
+        if (stack == 0){
+          local_results[i] = GCCMSingle4GridOneDim(
+            Emb2D,
+            yPred,
+            lib_size,
+            lib,
+            pred,
+            totalRow,
+            totalCol,
+            b,
+            simplex,
+            theta,
+            threads_sizet,
+            parallel_level,
+            dist_metric,
+            dist_average
+          );
+        } else {
+          local_results[i] = GCCMSingle4GridOneDim(
+            Emb3D,
+            yPred,
+            lib_size,
+            lib,
+            pred,
+            totalRow,
+            totalCol,
+            b,
+            simplex,
+            theta,
+            threads_sizet,
+            parallel_level,
+            dist_metric,
+            dist_average
+          );
+        }
       }, threads_sizet);
     }
   }
