@@ -51,6 +51,22 @@ std::vector<std::pair<int, double>> GCCMSingle4Lattice(
     bool dist_average
 );
 
+// Perform GCCM on a single lib and pred for lattice data (composite embeddings version).
+std::vector<std::pair<int, double>> GCCMSingle4Lattice(
+    const std::vector<std::vector<std::vector<double>>>& x_vectors,
+    const std::vector<double>& y,
+    int lib_size,
+    const std::vector<int>& lib_indices,
+    const std::vector<int>& pred_indices,
+    int b,
+    bool simplex,
+    double theta,
+    size_t threads,
+    int parallel_level,
+    int dist_metric,
+    bool dist_average
+);
+
 /**
  * Performs GCCM on a spatial lattice data.
  *
@@ -69,6 +85,7 @@ std::vector<std::pair<int, double>> GCCMSingle4Lattice(
  * - threads: Number of threads to use for parallel computation.
  * - parallel_level: Level of parallel computing: 0 for `lower`, 1 for `higher`.
  * - style: Embedding style selector (0: includes current state, 1: excludes it).
+ * - stack: Embedding arrangement selector (0: single - average lags, 1: composite - stack).  Default is 0 (average lags).
  * - dist_metric: Distance metric selector (1: Manhattan, 2: Euclidean).
  * - dist_average: Whether to average distance by the number of valid vector components.
  * - single_sig: Whether to estimate significance and confidence intervals using a single rho value.
@@ -79,8 +96,8 @@ std::vector<std::pair<int, double>> GCCMSingle4Lattice(
  *      - The library size.
  *      - The mean cross-mapping correlation.
  *      - The statistical significance of the correlation.
- *      - The upper bound of the confidence interval.
  *      - The lower bound of the confidence interval.
+ *      - The upper bound of the confidence interval.
  */
 std::vector<std::vector<double>> GCCM4Lattice(
     const std::vector<double>& x,
@@ -97,6 +114,7 @@ std::vector<std::vector<double>> GCCM4Lattice(
     int threads,
     int parallel_level,
     int style,
+    int stack,
     int dist_metric,
     bool dist_average,
     bool single_sig,
