@@ -12,7 +12,7 @@
 #include "CppLatticeUtils.h"
 #include "SimplexProjection.h"
 #include "SMap.h"
-#include "spEDMDataStruct.h"
+#include "DataStruct.h"
 #include <RcppThread.h>
 
 /**
@@ -53,6 +53,26 @@ std::vector<double> PartialSimplex4Lattice(
     bool dist_average = true
 );
 
+/**
+ * Computes the partial correlation between the target variable and its simplex projection,
+ * incorporating control variables using a lattice-based embedding approach (composite embeddings version).
+ */
+std::vector<double> PartialSimplex4Lattice(
+    const std::vector<std::vector<std::vector<double>>>& vectors,
+    const std::vector<double>& target,
+    const std::vector<std::vector<double>>& controls,
+    const std::vector<std::vector<int>>& nb_vec,
+    const std::vector<int>& lib_indices,
+    const std::vector<int>& pred_indices,
+    const std::vector<int>& conEs,
+    const std::vector<int>& taus,
+    const std::vector<int>& num_neighbors,
+    bool cumulate = false,
+    int style = 1,
+    int dist_metric = 2,
+    bool dist_average = true
+);
+
 /**
  * @brief Computes the partial correlation between a spatial cross-sectional series and its prediction
  *        using the S-Map method, incorporating control variables.
@@ -96,6 +116,27 @@ std::vector<double> PartialSMap4Lattice(
     bool dist_average = true
 );
 
+/**
+ * Computes the partial correlation between a spatial cross-sectional series and its prediction
+ * using the S-Map method, incorporating control variables (composite embeddings version).
+ */
+std::vector<double> PartialSMap4Lattice(
+    const std::vector<std::vector<std::vector<double>>>& vectors,
+    const std::vector<double>& target,
+    const std::vector<std::vector<double>>& controls,
+    const std::vector<std::vector<int>>& nb_vec,
+    const std::vector<int>& lib_indices,
+    const std::vector<int>& pred_indices,
+    const std::vector<int>& conEs,
+    const std::vector<int>& taus,
+    const std::vector<int>& num_neighbors,
+    double theta = 1.0,
+    bool cumulate = false,
+    int style = 1,
+    int dist_metric = 2,
+    bool dist_average = true
+);
+
 /*
  * Perform SCPCM on a single library and prediction set for lattice data.
  *
@@ -146,6 +187,28 @@ std::vector<PartialCorRes> SCPCMSingle4Lattice(
     bool dist_average                                   // Whether to average distance by the number of valid vector components
 );
 
+// Perform SCPCM on a single library and prediction set for lattice data (composite embeddings version).
+std::vector<PartialCorRes> SCPCMSingle4Lattice(
+    const std::vector<std::vector<std::vector<double>>>& x_vectors,
+    const std::vector<double>& y,
+    const std::vector<std::vector<double>>& controls,
+    const std::vector<std::vector<int>>& nb_vec,
+    int lib_size,
+    const std::vector<int>& lib_indices,
+    const std::vector<int>& pred_indices,
+    const std::vector<int>& conEs,
+    const std::vector<int>& taus,
+    const std::vector<int>& b,
+    bool simplex,
+    double theta,
+    size_t threads,
+    int parallel_level,
+    bool cumulate,
+    int style,
+    int dist_metric,
+    bool dist_average
+);
+
 /**
  * Performs SCPCM on a spatial lattice dataset.
  *
@@ -166,6 +229,7 @@ std::vector<PartialCorRes> SCPCMSingle4Lattice(
  * - parallel_level: Level of parallel computing: 0 for `lower`, 1 for `higher`.
  * - cumulate: Boolean flag indicating whether to cumulate partial correlations.
  * - style: Embedding style selector (0: includes current state, 1: excludes it).
+ * - stack: Embedding arrangement selector (0: single - average lags, 1: composite - stack).  Default is 0 (average lags).
  * - dist_metric: Distance metric selector (1: Manhattan, 2: Euclidean).
  * - dist_average: Whether to average distance by the number of valid vector components.
  * - single_sig: Whether to estimate significance and confidence intervals using a single rho value.
@@ -176,12 +240,12 @@ std::vector<PartialCorRes> SCPCMSingle4Lattice(
  *      - The library size.
  *      - The mean pearson cross-mapping correlation.
  *      - The statistical significance of the pearson correlation.
- *      - The upper bound of the pearson correlation confidence interval.
  *      - The lower bound of the pearson correlation confidence interval.
+ *      - The upper bound of the pearson correlation confidence interval.
  *      - The mean partial cross-mapping partial correlation.
  *      - The statistical significance of the partial correlation.
- *      - The upper bound of the partial correlation confidence interval.
  *      - The lower bound of the partial correlation confidence interval.
+ *      - The upper bound of the partial correlation confidence interval.
  */
 std::vector<std::vector<double>> SCPCM4Lattice(
     const std::vector<double>& x,                       // Spatial cross-sectional series to cross map from
@@ -200,6 +264,7 @@ std::vector<std::vector<double>> SCPCM4Lattice(
     int parallel_level,                                 // Level of parallel computing: 0 for `lower`, 1 for `higher`
     bool cumulate,                                      // Whether to cumulate the partial correlations
     int style,                                          // Embedding style selector (0: includes current state, 1: excludes it)
+    int stack,                                          // Embedding arrangement selector (0: single - average lags, 1: composite - stack).  Default is 0 (average lags).
     int dist_metric,                                    // Distance metric selector (1: Manhattan, 2: Euclidean)
     bool dist_average,                                  // Whether to average distance by the number of valid vector components
     bool single_sig,                                    // Whether to estimate significance and confidence intervals using a single rho value
